#!/bin/bash

print_help() {
    cat >&2 << EOF
Usage: $(basename "$0") [OPTION]... [INPUTFILE]...
Build a self-extracting package for datasid-agent.

Options:
  -h, --help                    print this help and exit
  -o, --output=OUTPUTFILE       set the output package file
  -v, --set-version=VERSION     set the version number to use in the package
  -b, --bootstrap=BOOTSTRAP     select a file to use as bootstrap (default is
                                'bootstrap/install')

NOTE:
  OUTPUTFILE, VERSION and at least one INPUTFILE must be set!

Example of usage:
  $(basename "$0") --set-version 1.0.4 --output datasid-package.run \\
      bin/datasid-agent.sh bin/datasid-common.sh conf/dataseed.conf

EOF
}

parse_options() {
    opts="ho:v:b:"
    longopts="help,output:,set-version:,bootstrap:"

    ARGS=`getopt -o "$opts" -l "$longopts" -n "$(basename "$0")" -- "$@"`

    # bad arguments
    if [ $? -ne 0 ]; then
        echo >&2 "Try '$(basename "$0") --help' for more information."
        exit 1
    fi

    eval set -- "$ARGS"

    while true; do
        case "$1" in
            -h|--help)
                print_help
                exit 0
                shift;;

            -o|--output)
                output="$2"
                shift 2;;

            -v|--set-version)
                version="$2"
                shift 2;;

            -b|--bootstrap)
                bootstrap="$2"
                shift 2;;

            --)
                shift
                break;;
        esac
    done

    if [ "$#" -lt 1 ]; then
        echo >&2 "No input file."
        echo >&2 "Try '$(basename "$0") --help' for more information."
        exit 2
    fi

    if [ -z "$output" ]; then
        echo >&2 "No output file specified."
        echo >&2 "Try '$(basename "$0") --help' for more information."
        exit 3
    fi

    if [ -z "$version" ]; then
        echo >&2 "No version number specified."
        echo >&2 "Try '$(basename "$0") --help' for more information."
        exit 4
    fi

    input=$@
}

# global variables
output=
version=
input=
bootstrap="bootstrap/bootstrap-install"

# parse command line options into the global variables above
parse_options "$@"

set -o xtrace
set -o errexit

# create a temporary directory
tempdir=$(mktemp -d /tmp/datasid.XXXXXXXX)

# copy input files to a temporary directory and calc md5sum
cp --parents -a $input "$tempdir"
md5sum $input > "$tempdir/.md5sum"

# create version file
mkdir -p $tempdir/conf
echo "$version" > "$tempdir/conf/version"

mkdir -p $tempdir/data

# create a gzipped tar with the files
tempfile=$(mktemp /tmp/datasid.XXXXXXXX)
tar --directory "$tempdir" -czf "$tempfile" .

rm -rf "$tempdir"

pkgmd5=$(md5sum "$tempfile" | cut -d' ' -f1)
pkgline=$(( $(wc -l "$bootstrap" | cut -d' ' -f1) + 1 ))

sed "s/PKG_LINE=/PKG_LINE=${pkgline}/g; \
     s/PKG_MD5=/PKG_MD5=\"${pkgmd5}\"/g" \
    "$bootstrap" > "$output"

cat "$tempfile" >> "$output"
chmod +x "$output"

rm -f "$tempfile"

set +o xtrace
set +o errexit

exit 0
